﻿define("epi-cms/project/ProjectComponentViewModel", [
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Deferred",
    "dojo/Evented",
    "dojo/Stateful",
    "dojo/topic",
    "dojo/dom-class",
    "dojo/promise/all",
    "dojo/when",

// dijit
    "dijit/Destroyable",

//epi
    "epi/epi",
    "epi/datetime",
    "epi/dependency",
    "epi/username",
    "epi/shell/_ContextMixin",
    "epi/shell/xhr/ErrorHandler",
    "epi-cms/contentediting/ContentActionSupport",

// Commands
    "epi/shell/command/withConfirmation",
    "./command/AddProject",
    "./command/PublishProject",
    "./command/EditProjectItem",
    "./command/ReadyToPublishProjectItem",
    "./command/RefreshProjectItems",
    "./command/RemoveProject",
    "./command/RemoveProjectItem",
    "./command/RemoveProjectScheduling",
    "./command/RenameProject",
    "./command/SortProjectItems",
    "./command/ScheduleProject",

// Resources
    "epi/i18n!epi/nls/episerver.cms.components.project",
    "epi/i18n!epi/nls/episerver.shared.action"
],

function (
// dojo
    declare,
    lang,
    Deferred,
    Evented,
    Stateful,
    topic,
    domClass,
    all,
    when,

// dijit
    Destroyable,

// epi
    epi,
    epiDate,
    dependency,
    username,
    _ContextMixin,
    ErrorHandler,
    ContentActionSupport,

// Commands
    withConfirmation,
    AddProject,
    PublishProject,
    EditProjectItem,
    ReadyToPublishProjectItem,
    RefreshProjectItems,
    RemoveProject,
    RemoveProjectItem,
    RemoveProjectScheduling,
    RenameProject,
    SortProjectItems,
    ScheduleProject,

// Resources
    res,
    resShared
) {
    return declare([Stateful, Destroyable, Evented, _ContextMixin], {
        // summary:
        //      The view model for the epi-cms/project/ProjectComponent
        // tags:
        //      internal

        // commands: [readonly] epi/shell/command/_Command[]
        //      Commands to be consumed by the view.
        commands: null,

        // namedCommands: [readonly] Object
        //      Way to access commands using named keys.
        namedCommands: null,

        // created: [readonly] string
        //      A view prepared version of the date the selected project was created.
        created: "",

        // createdBy: [readonly] string
        //      A view prepared version of the user that created the selected project.
        createdBy: "",

        // profile: [readonly] Profile
        //      The current user profile
        profile: null,

        // createdProjectInfo: [readonly] string
        //      A view prepared version of the placeholder info.
        createProjectInfo: "",

        // contentLanguage: [readonly] string
        //      The current UI language
        contentLanguage: "",

        dndEnabled: false,

        // hasProject: [public] bool
        //      True if we have any projects
        hasProject: false,

        // projectStore: [readonly] Store
        //      A REST store for interacting with projects.
        projectStore: null,

        // projectSortOrder: [public] Object
        //      The order projects are sorted by.
        projectSortOrder: null,

        // projectItemStore: [readonly] Store
        //      A REST store for interacting with project items.
        projectItemStore: null,

        // projectItemQuery: Query
        //      Query object holding parameters to get a projects items.
        projectItemQuery: null,

        // projectItemSortOrder: [public] Object
        //      The order to sort project items by.
        projectItemSortOrder: null,

        // projectStatus: [public] string
        //      State that indicates if a project and it's contents are available.
        projectStatus: "",

        // notificationMessage: [public] string
        //      The message to show in the notification bar.
        notificationMessage: "",

        // selectedProject: [public] Object
        //      The project that is currently selected.
        selectedProject: null,

        // selectedProjectItem: [public] Object
        //      The project item that is currently selected.
        selectedProjectItem: null,

        // placeholderState: [public] string
        //      State that indicates if a project and it's contents are available.
        placeholderState: null,

        _messages: null,

        _selectedProjectProfileKey: "epi.selected-project-id",

        _sortOrderProfileKey: "epi.project-sort-order",

        constructor: function() {
            // Initialize default values for object typed properties.
            this.projectItemSortOrder = [];
            this.projectSortOrder = [{ attribute: "created", descending: true }];
        },

        postscript: function () {
            this.inherited(arguments);

            // Resolve the stores from the registry if they haven't been injected. This allows
            // us to mock the stores when testing.
            this.projectService = this.projectService || dependency.resolve("epi.cms.ProjectService");
            this.projectStore = this.projectStore || dependency.resolve("epi.storeregistry").get("epi.cms.project");
            this.projectItemStore = this.projectItemStore || dependency.resolve("epi.storeregistry").get("epi.cms.project.item");
            this.profile = this.profile || dependency.resolve("epi.shell.Profile");

            this._messages = this.isProjectModeEnabled() ? res.status.message : lang.mixin({}, res.status.message, res.status["messagelegacy"]);

            this._createCommands();
        },

        initialize: function () {
            // summary:
            //
            // returns: Promise
            //
            // tags:
            //      public

            return all(
                // Set the sort order based on what is stored in the profile, fall back to default value if nothing stored.
                when(this.profile.get(this._sortOrderProfileKey), lang.hitch(this, function (sort)  {
                    var option = this._findSortOption("key", sort) || this.namedCommands.sortProjectItems.options[0]; // default to name ascending
                    this._changeAttrValue("projectItemSortOrder", option.value); // use the internal setter to not trigger a profile save
                })),
                // Load the selected project from the profile and update hasProject.
                when(this._loadSelectedProject(), lang.hitch(this, function (project) {

                    // if a project has been resolved we know that there is at least one available
                    // so that we can pass it along to the hasProject check
                    var hasProject = !!project || undefined;

                    this._updateSelectedProjectDependencies(project);
                    this._changeAttrValue("selectedProject", project);  // use the internal setter to not trigger a profile save

                    return this._updateHasProject(hasProject);
                })),
                // Get the current content language from the profile
                when(this.profile.getContentLanguage(), lang.hitch(this, function (contentLanguage) {
                    this.set("contentLanguage", contentLanguage);
                }))
            );
        },

        isProjectModeEnabled: function() {
            // summary:
            //      Indicates whether project mode is enabled or not
            // tags:
            //      public

            return this.projectService.isProjectModeEnabled;
        },

        getCommands: function () {
            // summary:
            //      Returns all available commands
            // tags:
            //      public

            return this.commands;
        },

        addProject: function (project) {
            // summary:
            //      Adds a project and sets it as the selectedProject
            // tags:
            //      public

            this.projectStore.add(project)
                .then(lang.hitch(this, function (value) { this.set("selectedProject", value); }))
                .otherwise(ErrorHandler.forXhr);
        },

        updateProject: function (project) {
            // summary:
            //      Adds a project and sets it as the selectedProject
            // tags:
            //      public

            this.projectStore.put(project)
                .then(lang.hitch(this, "set", "selectedProject"))
                .otherwise(ErrorHandler.forXhr);
        },

        publishProject: function (projectId, delayPublishUntil) {
            // summary:
            //      Publishes a project with given id right away or delay publishing until given delayPublish date.
            // projectId: Number
            //      Id of the project to publish
            // delayPublishUntil: Date?
            //      If supplied, then the project will be delay published.
            // returns: Promise
            // tags:
            //      public

            var self = this;

            return this.projectService.publishProject(projectId, delayPublishUntil)
                .then(function (updatedProject) {
                    return self._updateSelectedProjectAndRefreshContextRequest(updatedProject)
                        .then(function () {
                            return updatedProject;
                        });
                });
        },

        reactivateProject: function (projectId) {
            // summary:
            //      Reactivates a project that has been scheduled to publish.
            // projectId: Number
            //      The ID of the project to reactivate
            // returns: Promise
            // tags:
            //      public

            var self = this;

            if (isNaN(projectId)) {
                throw new Error("The given project ID is not a number.");
            }

            return this.projectService.reactivateProject(projectId)
                .then(function (updatedProject) {
                    return self._updateSelectedProjectAndRefreshContextRequest(updatedProject)
                        .then(function () {
                            return updatedProject;
                        });
                });
        },

        refreshProject: function () {
            // summary:
            //      Refreshes the selected project, particularly information if the project can be published or not.
            // tags:
            //      public

            var self = this;

            if (!this.selectedProject) {
                this._updateHasProject();
                this.emit("refresh");
                return;
            }

            return this.projectStore.refresh(this.selectedProject.id).then(
                function (value) {
                    self.set("selectedProject", value);
                    self.emit("refresh");
                    return value;
                },
                function (ex) {
                    self.set("selectedProject", null);
                }
            );
        },

        removeProject: function () {
            // summary:
            //      Removes the selected project and sets the selected project property to null.
            // tags:
            //      public
            var self = this,
                store = this.projectStore;

            if (!this.selectedProject) {
                return new Deferred().resolve();
            }

            return store.remove(this.selectedProject.id).then(function () {
                self.set("selectedProject", null);
            });
        },

        canAddContent: function (contentReferences) {
            // summary:
            //      Verifies that the given content references can be added to the currently selected project
            // contentReferences: Array
            //      The content references to add to the currently selected project
            // returns: Promise
            //      A promise
            // tags:
            //      public

            var selectedProjectId = this.get("selectedProject").id;

            return this.projectService.canAddContent(selectedProjectId, contentReferences);
        },

        canPublishProject: function (project) {
            // summary:
            //      Checks if a project can be published. Uses either a project
            //      as an argument or the currently selected proejct.
            // tags:
            //      public

            var itemStatusCount;

            function noPublishAccess() {
                return Object.keys(itemStatusCount).some(function(key) {
                    if(key.indexOf("nopublishaccess") > -1) {
                        if(itemStatusCount[key] > 0) {
                            return true;
                        }
                    }
                });
            }

            function checkState(state) {
                return itemStatusCount[state] > 0;
            }

            project = project || this.selectedProject;
            itemStatusCount = project && project.itemStatusCount;

            if (this.isProjectModeEnabled()) {
                return !!project && project.itemStatusCount.checkedin > 0;
            } else {
                return !!project && (project.status === "active" || project.status === "publishfailed") &&
                       !noPublishAccess() && !checkState("notcreated") && !checkState("rejected") && !checkState("checkedout") &&
                       (checkState("checkedin") || checkState("published") || checkState("previouslypublished") || checkState("delayedpublish"));
            }
        },

        addProjectItems: function (contentReferences, projectId) {
            // summary:
            //      Adds the given content references as items of the project.
            // tags:
            //      public

            var selectedProject = this.get("selectedProject");
            projectId = projectId || (selectedProject && selectedProject.id);

            if (!projectId) {
                throw new Error("If no projectId is provided a selectedProject must be set in the context");
            }

            return this.projectService.addProjectItems(projectId, contentReferences)
                .then(lang.hitch(this, "emit", "refresh"));
        },

        removeSelectedProjectItem: function () {
            // summary:
            //      Removes a project item from a project.
            // tags:
            //      public

            var projectId = this.get("selectedProject").id;

            this.projectService.removeProjectItems(projectId, [this.selectedProjectItem])
                .then(lang.hitch(this, "refreshProject"));
        },

        markProjectItemAsReadyToPublish: function () {
            // summary:
            //      Mark the project item as ready to publish
            // returns: Promise
            //      A promise that resolves when the refresh project has completed.
            // tags:
            //      public

            var self = this;
            var promise = this.projectService.markAsReadyToPublish(this.selectedProjectItem.id)
                .then(lang.hitch(this, "refreshProject"))
                .then(function () {
                    when(self.getCurrentContext()).then(function(ctx) {
                        if (ctx.id === self.selectedProjectItem.contentLink) {
                            self.requestContextChange(ctx.id);
                            return ctx;
                        }
                    });
                });

            return promise;
        },

        requestContextChange: function (contextOrContentReference) {
            // summary:
            //      Requests that the UI changes the context to the specified contentReference.
            // tags:
            //      public

            var context = contextOrContentReference.uri ? contextOrContentReference :
                { uri: "epi.cms.contentdata:///" +  contextOrContentReference};

            topic.publish("/epi/shell/context/request", context, { sender: this });
        },

        _createCommands: function () {

            var commandArgs = { model: this },
                commands,
                namedCommands = {
                    refreshProjectItems: new RefreshProjectItems(commandArgs),
                    addProject: new AddProject(commandArgs),
                    publishProject:  new PublishProject(commandArgs),
                    editProjectItem: new EditProjectItem(commandArgs),
                    readyToPublishProjectItem: new ReadyToPublishProjectItem(commandArgs),
                    removeProject: new RemoveProject({ model: this, store: this.projectStore }),
                    removeProjectItem: withConfirmation(new RemoveProjectItem({ model: this }), null, {
                        title: res.command.removeprojectitem.label,
                        description: res.command.removeprojectitem.confirmation,
                        confirmActionText: resShared.remove,
                        cancelActionText: resShared.cancel
                    }),
                    removeProjectScheduling: withConfirmation(new RemoveProjectScheduling(commandArgs), null, {
                        title: res.command.removeschedulingprojectitem.title,
                        description: res.command.removeschedulingprojectitem.confirmation,
                        confirmActionText: resShared.remove,
                        cancelActionText: resShared.cancel
                        }),
                    renameProject: new RenameProject(commandArgs),
                    scheduleProject: new ScheduleProject(commandArgs),
                    sortProjectItems: new SortProjectItems(commandArgs)
                };

            commands = this.own(
                namedCommands.sortProjectItems,
                namedCommands.refreshProjectItems,
                namedCommands.addProject,
                namedCommands.renameProject,
                namedCommands.publishProject,
                namedCommands.removeProject,
                namedCommands.readyToPublishProjectItem,
                namedCommands.editProjectItem,
                namedCommands.removeProjectItem,
                namedCommands.scheduleProject,
                namedCommands.removeProjectScheduling
            );

            this.set({
                commands: commands,
                namedCommands: namedCommands
            });
        },

        _selectNextProject: function () {
            // summary:
            //      Selects next project
            // tags:
            //      private

            when(this._getProjects(0, 1, this.get("projectSortOrder")), lang.hitch(this, function (results) {
                var value = results.length > 0 ? results[0] : null;
                this.set("selectedProject", value);
                this.set("hasProject", results.length > 0);
            }));
        },

        _getProjects: function (start, count, sort) {
            // summary:
            //      Gets projects based on attributes
            // tags:
            //      private
            return this.projectStore.query(null, { start: start, count: count, sort: sort });
        },

        _updateHasProject: function (hasProject) {
            // summary:
            //      updates hasProject boolean depending on getHasProject result
            // tags:
            //      private

            return when(hasProject === true ? [hasProject] : this._getProjects(0, 1), lang.hitch(this, function (results) {
                var result = results.length > 0;
                this.set("hasProject", result);

                return new Deferred().resolve(result).promise;
            }));
        },

        _loadSelectedProject: function () {
            // summary:
            //      Load the selected project from the user profile
            // returns: Promise
            //      The project that was stored in the profile
            // tags:
            //      protected

            var projectStore = this.projectStore;

            //Load the selected project from the profile
            return when(this.profile.get(this._selectedProjectProfileKey), function (projectId) {
                if (!projectId) {
                    return null;
                }

                // Need to get the project from the store in order to ensure it still exists.
                // Otherwise return null
                return when(projectStore.get(projectId)).otherwise(function (ex) { return null; });
            });
        },

        _persistSelectedProject: function (selectedProject) {
            // summary:
            //      Store the selected project in the user profile
            //  tags: protected

            this.profile.set(this._selectedProjectProfileKey, selectedProject && selectedProject.id, { location: "server" });
        },

        _persistSortOrder: function (sortOrder) {
            // summary:
            //      Store the project item sort order in the user profile
            //  tags: protected

            var option = this._findSortOption("value", sortOrder),
                key = option && option.key || null;

            this.profile.set(this._sortOrderProfileKey, key, { location: "server" });
        },

        _findSortOption: function (key, term) {

            var options = this.namedCommands.sortProjectItems.options,
                option = null;

            term && options.some(function (item) {
                if (term === item[key]) {
                    option = item;
                    return true;
                }
            });

            return option;
        },

        _updateSelectedProjectDependencies: function (selectedProject) {

            var created = "",
                createdBy = "",
                delayPublish = "",
                dndEnabled = false,
                failedItemsCount = 0,
                notificationMessage = "",
                projectItemQuery = null,
                projectStatus = "",
                publishfailed = res.notifications.publishfailed,
                isProjectModeEnabled = this.isProjectModeEnabled();

            function getUnpublishedItems(statuses) {
                var count = 0;

                Object.keys(statuses).forEach(function(key) {
                    if (key.indexOf("checkedin") === 0) {
                        count += statuses[key];
                    }
                });
                return count;
            }

            if (selectedProject) {
                created = epiDate.toUserFriendlyString(epiDate.deserialize(selectedProject.created));
                createdBy = username.toUserFriendlyString(selectedProject.createdBy, null, true);
                dndEnabled = (isProjectModeEnabled && selectedProject.status !== "publishing") || selectedProject.status === "active" || selectedProject.status === "publishfailed";
                projectItemQuery = { projectId: selectedProject.id };

                if (!this.canPublishProject(selectedProject) || isProjectModeEnabled) {
                    projectStatus = this._messages[selectedProject.status] || "";

                    if (selectedProject.status === "delayedpublished" && !isProjectModeEnabled) {
                        delayPublish = epiDate.toUserFriendlyString(selectedProject.delayPublishUntil);
                        projectStatus = lang.replace(projectStatus, { date: delayPublish });
                        notificationMessage = lang.replace(res.status.state.delayedpublished, { delayPublishUntil: delayPublish });
                    }
                }

                // If the project failed to publish then update the notification bar message.
                if (selectedProject.status === "publishfailed") {
                    failedItemsCount = getUnpublishedItems(selectedProject.itemStatusCount);

                    if (failedItemsCount > 0) {
                        notificationMessage = lang.replace(failedItemsCount > 1
                            ? publishfailed.notificationtextmultipleitems
                            : publishfailed.notificationtextsingleitem, { failedItemsCount: failedItemsCount });
                    }
                }
            }

            this.set({
                created: created,
                createdBy: createdBy,
                dndEnabled: dndEnabled,
                notificationMessage: notificationMessage,
                projectItemQuery: projectItemQuery, // Refresh the list query when the selected project changes.
                projectStatus: projectStatus
            });
        },

        _updatePlaceholderState: function () {

            var hasProjects = this.get("hasProject"),
                hasSelectedProject = !!this.get("selectedProject"),
                state = "";

            if (!hasProjects) {
                // should set placeholderState to 'noProjects' when there are no projects
                state = "noProjects";
                this.set("createProjectInfo", res.placeholder.noprojectscreated);
            } else if (hasProjects && !hasSelectedProject) {
                // should set placeholderState to 'noSelectedProject' when there are projects available and no project is selected
                state = "noSelectedProject";
                this.set("createProjectInfo", res.placeholder.noprojectselected);
            }

            this.set("placeholderState", state);
        },

        _hasProjectSetter: function (value) {
            this.hasProject = value;
            this._updatePlaceholderState();
        },

        _selectedProjectSetter: function (selectedProject) {

            //Store return the same instance sometimes.
            selectedProject = lang.clone(selectedProject);

            // Early exit if the project has not changed
            if (epi.areEqual(this.selectedProject, selectedProject)) { return; }

            this.selectedProject = selectedProject;
            this._updateSelectedProjectDependencies(selectedProject);

            if(!this.hasProject && selectedProject) {
                this.set("hasProject", true);
            } else {
                this._updatePlaceholderState();
            }

            !selectedProject && this._selectNextProject();

            this._persistSelectedProject(selectedProject);
        },

        _projectItemSortOrderSetter: function (sortOrder) {
            // summary:
            //      Store the selected project in the user profile

            if (sortOrder && sortOrder === this.projectItemSortOrder) { return; }

            this.projectItemSortOrder = sortOrder;

            this._persistSortOrder(sortOrder);

        },

        _updateSelectedProjectAndRefreshContextRequest: function(updatedProject) {
            // summary:
            //      Updates the selectedProject with given project and runs the context request change with currently available context
            // returns: Promise
            //      The context which is reloaded/refreshed.
            // tags:
            //      internal

            var self = this;

            // update the selected project and also change context request to current context.
            self.set("selectedProject", updatedProject);

            return when(this.getCurrentContext()).then(function(context) {
                    self.requestContextChange(context);

                    return context;
                }
            );
        }
    });
});