define("epi-cms/project/viewmodels/OverviewViewModel", [
	"dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/topic",
    "dojo/when",
    "epi/dependency",

	 "../ProjectComponentViewModel"
], function (
//dojo
	declare,
	lang,
	topic,
    when,

//epi
    dependency,
	ProjectComponentViewModel
) {
	return declare([ProjectComponentViewModel], {
        // summary:
        //
        // tags:
        //      internal

        // contextHistory: [readonly] epi-cms/BackContextHistory
        //      The context history stack
        contextHistory: null,

        // isProjectOverviewActive: [public] Boolean
        //      Indicates if the overview is open
        isProjectOverviewActive: false,

		postscript: function () {
			this.inherited(arguments);

            this.contextHistory = this.contextHistory || dependency.resolve("epi.cms.BackContextHistory");
			this.own(this.projectService.on("currentProjectChanged", lang.hitch(this, this.projectOverviewChanged)));
		},

        contextChanged: function (/*Object*/context, /*Object*/callerData) {
            // summary:
            //      When context changes
            // tags:
            //      protected 
            this.inherited(arguments);

            if(context && context.type === "epi.cms.project") {
                when(this.projectStore.refresh(context.id), lang.hitch(this,function(currentProject) {
                    this.set("selectedProject", currentProject);
                    this.emit("refresh");

                }));
            }
        },

        projectOverviewChanged: function (project) {
            // summary:
            //      When the project overview changes
            // project: Object
            //      Project      
            // tags:
            //      public   
                 
            var active = this.get("isProjectOverviewActive"),
                newProjectUri;

            // id is empty when selecting None.
            if(active) {
                if (project && project.id) {
                    newProjectUri = "epi.cms.project:///" + project.id;
                    topic.publish("/epi/shell/context/request", { uri: newProjectUri }, { sender: this });
                } else {
                    this.requestPreviousContext();
                }
            }
        },

        requestPreviousContext: function () {
            // summary:
            //      Requests the previous context and publish context change
            // tags:
            //      public  
            var context = this.contextHistory && this.contextHistory.getPreviousContent();
            if (context && context.uri) {
                topic.publish("/epi/shell/context/request", context, { sender: this });
            } 
        },

        _loadSelectedProject: function () {
            // summary:
            //      Loads the selected project
            // returns: Promise
            //      The project that was stored in the profile
            // tags:
            //      protected

            return this.projectService.getCurrentProject();
        },

        _persistSelectedProject: function (selectedProject) {
            // summary:
            //      The overview has no interest in storing selected projects
            //      since the context is in charge of which project to show.
            //  tags: protected override
        }
	});
});